:root {
    --body-text-color: #FFFFFF;
    --link-text-color: #FFFFFF;
    --link-text-color-active: #FFFFFF;
    --link-text-color-hover: #FFFFFF;
    --link-text-color-visited: #FFFFFF;
    --body-text-color-subdued: #FFFFFF;
    --block-info-text-color: #FFFFFF;
    --block-label-text-color: #FFFFFF;
    --block-title-text-color: #FFFFFF;
    --checkbox-label-text-color: #FFFFFF;
    --checkbox-label-text-color-selected: #FFFFFF;
    --error-text-color: #FFFFFF;
    --button-cancel-text-color: #FFFFFF;
    --button-cancel-text-color-hover: #FFFFFF;
    --button-primary-text-color: #FFFFFF;
    --button-primary-text-color-hover: #FFFFFF;
    --button-secondary-text-color: #FFFFFF;
    --button-secondary-text-color-hover: #FFFFFF;
    

    --border-bottom-right-radius: 0px;
    --border-bottom-left-radius:  0px;
    --border-top-right-radius:    0px;
    --border-top-left-radius:     0px;
    --block-radius:     0px;
    --button-large-radius: 0px;
    --button-small-radius: 0px;
    --block-background-fill: #000000;

    --border-color-accent: #3cff00;
    --border-color-primary: #3cff00;
    --block-border-color: #3cff00;
    --block-label-border-color: #3cff00;
    --block-title-border-color: #3cff00;
    --panel-border-color: #3cff00;
    --checkbox-border-color: #3cff00;
    --checkbox-border-color-focus: #3cff00;
    --checkbox-border-color-hover: #3cff00;
    --checkbox-border-color-selected: #3cff00;
    --checkbox-label-border-color: #3cff00;
    --checkbox-label-border-color-hover: #3cff00;
    --error-border-color: #3cff00;
    --input-border-color: #3cff00;
    --input-border-color-focus: #3cff00;
    --input-border-color-hover: #3cff00;
    --table-border-color: #3cff00;
    --button-cancel-border-color: #3cff00;
    --button-cancel-border-color-hover: #3cff00;
    --button-primary-border-color: #3cff00;
    --button-primary-border-color-hover: #3cff00;
    --button-secondary-border-color: #3cff00;
    --button-secondary-border-color-hover: #3cff00;
    
    
    --body-background-fill: #000000;
    --background-fill-primary: #000000;
    --background-fill-secondary: #000000;
    --block-background-fill: #000000;
    --block-label-background-fill: #000000;
    --block-title-background-fill: #000000;
    --panel-background-fill: #000000;
    --chatbot-code-background-color: #000000;
    --checkbox-background-color: #000000;
    --checkbox-background-color-focus: #000000;
    --checkbox-background-color-hover: #000000;
    --checkbox-background-color-selected: #000000;
    --checkbox-label-background-fill: #000000;
    --checkbox-label-background-fill-hover: #000000;
    --checkbox-label-background-fill-selected: #000000;
    --error-background-fill: #000000;
    --input-background-fill: #000000;
    --input-background-fill-focus: #000000;
    --input-background-fill-hover: #000000;
    --stat-background-fill: #000000;
    --table-even-background-fill: #000000;
    --table-odd-background-fill: #000000;
    --button-cancel-background-fill: #000000;
    --button-cancel-background-fill-hover: #000000;
    --button-primary-background-fill: #000000;
    --button-primary-background-fill-hover: #000000;
    --button-secondary-background-fill: #000000;
    --button-secondary-background-fill-hover: #000000;
    --color-accent-soft: #000000;
}

.dark {
    --body-text-color: #FFFFFF;
    --link-text-color: #FFFFFF;
    --link-text-color-active: #FFFFFF;
    --link-text-color-hover: #FFFFFF;
    --link-text-color-visited: #FFFFFF;
    --body-text-color-subdued: #FFFFFF;
    --block-info-text-color: #FFFFFF;
    --block-label-text-color: #FFFFFF;
    --block-title-text-color: #FFFFFF;
    --checkbox-label-text-color: #FFFFFF;
    --checkbox-label-text-color-selected: #FFFFFF;
    --error-text-color: #FFFFFF;
    --button-cancel-text-color: #FFFFFF;
    --button-cancel-text-color-hover: #FFFFFF;
    --button-primary-text-color: #FFFFFF;
    --button-primary-text-color-hover: #FFFFFF;
    --button-secondary-text-color: #FFFFFF;
    --button-secondary-text-color-hover: #FFFFFF;
    


    --border-bottom-right-radius: 0px;
    --border-bottom-left-radius:  0px;
    --border-top-right-radius:    0px;
    --border-top-left-radius:     0px;
    --block-radius:     0px;
    --button-large-radius: 0px;
    --button-small-radius: 0px;
    --block-background-fill: #000000;

    --border-color-accent: #3cff00;
    --border-color-primary: #3cff00;
    --block-border-color: #3cff00;
    --block-label-border-color: #3cff00;
    --block-title-border-color: #3cff00;
    --panel-border-color: #3cff00;
    --checkbox-border-color: #3cff00;
    --checkbox-border-color-focus: #3cff00;
    --checkbox-border-color-hover: #3cff00;
    --checkbox-border-color-selected: #3cff00;
    --checkbox-label-border-color: #3cff00;
    --checkbox-label-border-color-hover: #3cff00;
    --error-border-color: #3cff00;
    --input-border-color: #3cff00;
    --input-border-color-focus: #3cff00;
    --input-border-color-hover: #3cff00;
    --table-border-color: #3cff00;
    --button-cancel-border-color: #3cff00;
    --button-cancel-border-color-hover: #3cff00;
    --button-primary-border-color: #3cff00;
    --button-primary-border-color-hover: #3cff00;
    --button-secondary-border-color: #3cff00;
    --button-secondary-border-color-hover: #3cff00;
    
    
    --body-background-fill: #000000;
    --background-fill-primary: #000000;
    --background-fill-secondary: #000000;
    --block-background-fill: #000000;
    --block-label-background-fill: #000000;
    --block-title-background-fill: #000000;
    --panel-background-fill: #000000;
    --chatbot-code-background-color: #000000;
    --checkbox-background-color: #000000;
    --checkbox-background-color-focus: #000000;
    --checkbox-background-color-hover: #000000;
    --checkbox-background-color-selected: #000000;
    --checkbox-label-background-fill: #000000;
    --checkbox-label-background-fill-hover: #000000;
    --checkbox-label-background-fill-selected: #000000;
    --error-background-fill: #000000;
    --input-background-fill: #000000;
    --input-background-fill-focus: #000000;
    --input-background-fill-hover: #000000;
    --stat-background-fill: #000000;
    --table-even-background-fill: #000000;
    --table-odd-background-fill: #000000;
    --button-cancel-background-fill: #000000;
    --button-cancel-background-fill-hover: #000000;
    --button-primary-background-fill: #000000;
    --button-primary-background-fill-hover: #000000;
    --button-secondary-background-fill: #000000;
    --button-secondary-background-fill-hover: #000000;
    --color-accent-soft: #000000;
}



.block.svelte-mppz8v {
    border-color: #3cff00;
}

/* 插件下拉菜单 */
#plugin-panel .wrap.svelte-aqlk7e.svelte-aqlk7e.svelte-aqlk7e {
    box-shadow: var(--input-shadow);
    border: var(--input-border-width) dashed var(--border-color-primary);
    border-radius: 4px;
}

#plugin-panel .dropdown-arrow.svelte-p5edak {
    width: 50px;
}
#plugin-panel input.svelte-aqlk7e.svelte-aqlk7e.svelte-aqlk7e {
    padding-left: 5px;
}
.root{
    border-bottom-right-radius: 0px;
    border-bottom-left-radius:  0px;
    border-top-right-radius:    0px;
    border-top-left-radius:     0px;
}

/* 小按钮 */
.sm {
    font-family: "Microsoft YaHei UI", "Helvetica", "Microsoft YaHei", "ui-sans-serif", "sans-serif", "system-ui";
    --button-small-text-weight: 600;
    --button-small-text-size:   16px;
    border-bottom-right-radius: 0px;
    border-bottom-left-radius:  0px;
    border-top-right-radius:    0px;
    border-top-left-radius:     0px;
}

#plugin-panel .sm {
    font-family: "Microsoft YaHei UI", "Helvetica", "Microsoft YaHei", "ui-sans-serif", "sans-serif", "system-ui";
    --button-small-text-weight: 400;
    --button-small-text-size:   14px;
    border-bottom-right-radius: 0px;
    border-bottom-left-radius:  0px;
    border-top-right-radius:    0px;
    border-top-left-radius:     0px;
}

.wrap-inner.svelte-aqlk7e.svelte-aqlk7e.svelte-aqlk7e {
    padding: 0%;
}

.markdown-body table {
    margin: 1em 0;
    border-collapse: collapse;
    empty-cells: show;
}

.markdown-body th, .markdown-body td {
    border: 1.2px solid var(--border-color-primary);
    padding: 5px;
}

.markdown-body thead {
    background-color: rgb(0, 0, 0);
}

.markdown-body thead th {
    padding: .5em .2em;
}

.normal_mut_select .svelte-1gfkn6j {
    float: left;
    width: auto;
    line-height: 260% !important;
}

.markdown-body ol, .markdown-body ul {
    padding-inline-start: 2em !important;
}

/* chat box. */
[class *= "message"] {
    border-radius: var(--radius-xl) !important;
    /* padding: var(--spacing-xl) !important; */
    /* font-size: var(--text-md) !important; */
    /* line-height: var(--line-md) !important; */
    /* min-height: calc(var(--text-md)*var(--line-md) + 2*var(--spacing-xl)); */
    /* min-width: calc(var(--text-md)*var(--line-md) + 2*var(--spacing-xl)); */
}
[data-testid = "bot"] {
    max-width: 95%;
    /* width: auto !important; */
    border-bottom-left-radius: 0 !important;
}
[data-testid = "user"] {
    max-width: 100%;
    /* width: auto !important; */
    border-bottom-right-radius: 0 !important;
}

/* linein code block. */
.markdown-body code {
    display: inline;
    white-space: break-spaces;
    border-radius: 6px;
    margin: 0 2px 0 2px;
    padding: .2em .4em .1em .4em;
    background-color: rgba(0, 0, 0, 0.95);
    color: #c9d1d9;
}

.dark .markdown-body code {
    display: inline;
    white-space: break-spaces;
    border-radius: 6px;
    margin: 0 2px 0 2px;
    padding: .2em .4em .1em .4em;
    background-color: rgba(0,0,0,0.2);
}

/* code block css */
.markdown-body pre code {
    display: block;
    overflow: auto;
    white-space: pre;
    background-color: rgba(0, 0, 0, 0.95);
    border-radius: 10px;
    padding: 1em;
    margin: 1em 2em 1em 0.5em;
}

.dark .markdown-body pre code {
    display: block;
    overflow: auto;
    white-space: pre;
    background-color: rgba(0,0,0,0.2);
    border-radius: 10px;
    padding: 1em;
    margin: 1em 2em 1em 0.5em;
}

/* .mic-wrap.svelte-1thnwz {

} */
.block.svelte-mppz8v > .mic-wrap.svelte-1thnwz{
    justify-content: center;
    display: flex;
    padding: 0;

}

.codehilite .hll { background-color: #6e7681 }
.codehilite .c { color: #8b949e; font-style: italic } /* Comment */
.codehilite .err { color: #f85149 } /* Error */
.codehilite .esc { color: #c9d1d9 } /* Escape */
.codehilite .g { color: #c9d1d9 } /* Generic */
.codehilite .k { color: #ff7b72 } /* Keyword */
.codehilite .l { color: #a5d6ff } /* Literal */
.codehilite .n { color: #c9d1d9 } /* Name */
.codehilite .o { color: #ff7b72; font-weight: bold } /* Operator */
.codehilite .x { color: #c9d1d9 } /* Other */
.codehilite .p { color: #c9d1d9 } /* Punctuation */
.codehilite .ch { color: #8b949e; font-style: italic } /* Comment.Hashbang */
.codehilite .cm { color: #8b949e; font-style: italic } /* Comment.Multiline */
.codehilite .cp { color: #8b949e; font-weight: bold; font-style: italic } /* Comment.Preproc */
.codehilite .cpf { color: #8b949e; font-style: italic } /* Comment.PreprocFile */
.codehilite .c1 { color: #8b949e; font-style: italic } /* Comment.Single */
.codehilite .cs { color: #8b949e; font-weight: bold; font-style: italic } /* Comment.Special */
.codehilite .gd { color: #ffa198; background-color: #490202 } /* Generic.Deleted */
.codehilite .ge { color: #c9d1d9; font-style: italic } /* Generic.Emph */
.codehilite .gr { color: #ffa198 } /* Generic.Error */
.codehilite .gh { color: #79c0ff; font-weight: bold } /* Generic.Heading */
.codehilite .gi { color: #56d364; background-color: #0f5323 } /* Generic.Inserted */
.codehilite .go { color: #8b949e } /* Generic.Output */
.codehilite .gp { color: #8b949e } /* Generic.Prompt */
.codehilite .gs { color: #c9d1d9; font-weight: bold } /* Generic.Strong */
.codehilite .gu { color: #79c0ff } /* Generic.Subheading */
.codehilite .gt { color: #ff7b72 } /* Generic.Traceback */
.codehilite .g-Underline { color: #c9d1d9; text-decoration: underline } /* Generic.Underline */
.codehilite .kc { color: #79c0ff } /* Keyword.Constant */
.codehilite .kd { color: #ff7b72 } /* Keyword.Declaration */
.codehilite .kn { color: #ff7b72 } /* Keyword.Namespace */
.codehilite .kp { color: #79c0ff } /* Keyword.Pseudo */
.codehilite .kr { color: #ff7b72 } /* Keyword.Reserved */
.codehilite .kt { color: #ff7b72 } /* Keyword.Type */
.codehilite .ld { color: #79c0ff } /* Literal.Date */
.codehilite .m { color: #a5d6ff } /* Literal.Number */
.codehilite .s { color: #a5d6ff } /* Literal.String */
.codehilite .na { color: #c9d1d9 } /* Name.Attribute */
.codehilite .nb { color: #c9d1d9 } /* Name.Builtin */
.codehilite .nc { color: #f0883e; font-weight: bold } /* Name.Class */
.codehilite .no { color: #79c0ff; font-weight: bold } /* Name.Constant */
.codehilite .nd { color: #d2a8ff; font-weight: bold } /* Name.Decorator */
.codehilite .ni { color: #ffa657 } /* Name.Entity */
.codehilite .ne { color: #f0883e; font-weight: bold } /* Name.Exception */
.codehilite .nf { color: #d2a8ff; font-weight: bold } /* Name.Function */
.codehilite .nl { color: #79c0ff; font-weight: bold } /* Name.Label */
.codehilite .nn { color: #ff7b72 } /* Name.Namespace */
.codehilite .nx { color: #c9d1d9 } /* Name.Other */
.codehilite .py { color: #79c0ff } /* Name.Property */
.codehilite .nt { color: #7ee787 } /* Name.Tag */
.codehilite .nv { color: #79c0ff } /* Name.Variable */
.codehilite .ow { color: #ff7b72; font-weight: bold } /* Operator.Word */
.codehilite .pm { color: #c9d1d9 } /* Punctuation.Marker */
.codehilite .w { color: #6e7681 } /* Text.Whitespace */
.codehilite .mb { color: #a5d6ff } /* Literal.Number.Bin */
.codehilite .mf { color: #a5d6ff } /* Literal.Number.Float */
.codehilite .mh { color: #a5d6ff } /* Literal.Number.Hex */
.codehilite .mi { color: #a5d6ff } /* Literal.Number.Integer */
.codehilite .mo { color: #a5d6ff } /* Literal.Number.Oct */
.codehilite .sa { color: #79c0ff } /* Literal.String.Affix */
.codehilite .sb { color: #a5d6ff } /* Literal.String.Backtick */
.codehilite .sc { color: #a5d6ff } /* Literal.String.Char */
.codehilite .dl { color: #79c0ff } /* Literal.String.Delimiter */
.codehilite .sd { color: #a5d6ff } /* Literal.String.Doc */
.codehilite .s2 { color: #a5d6ff } /* Literal.String.Double */
.codehilite .se { color: #79c0ff } /* Literal.String.Escape */
.codehilite .sh { color: #79c0ff } /* Literal.String.Heredoc */
.codehilite .si { color: #a5d6ff } /* Literal.String.Interpol */
.codehilite .sx { color: #a5d6ff } /* Literal.String.Other */
.codehilite .sr { color: #79c0ff } /* Literal.String.Regex */
.codehilite .s1 { color: #a5d6ff } /* Literal.String.Single */
.codehilite .ss { color: #a5d6ff } /* Literal.String.Symbol */
.codehilite .bp { color: #c9d1d9 } /* Name.Builtin.Pseudo */
.codehilite .fm { color: #d2a8ff; font-weight: bold } /* Name.Function.Magic */
.codehilite .vc { color: #79c0ff } /* Name.Variable.Class */
.codehilite .vg { color: #79c0ff } /* Name.Variable.Global */
.codehilite .vi { color: #79c0ff } /* Name.Variable.Instance */
.codehilite .vm { color: #79c0ff } /* Name.Variable.Magic */
.codehilite .il { color: #a5d6ff } /* Literal.Number.Integer.Long */

.dark .codehilite .hll { background-color: #2C3B41 }
.dark .codehilite .c { color: #79d618; font-style: italic } /* Comment */
.dark .codehilite .err { color: #FF5370 } /* Error */
.dark .codehilite .esc { color: #89DDFF } /* Escape */
.dark .codehilite .g { color: #EEFFFF } /* Generic */
.dark .codehilite .k { color: #BB80B3 } /* Keyword */
.dark .codehilite .l { color: #C3E88D } /* Literal */
.dark .codehilite .n { color: #EEFFFF } /* Name */
.dark .codehilite .o { color: #89DDFF } /* Operator */
.dark .codehilite .p { color: #89DDFF } /* Punctuation */
.dark .codehilite .ch { color: #79d618; font-style: italic } /* Comment.Hashbang */
.dark .codehilite .cm { color: #79d618; font-style: italic } /* Comment.Multiline */
.dark .codehilite .cp { color: #79d618; font-style: italic } /* Comment.Preproc */
.dark .codehilite .cpf { color: #79d618; font-style: italic } /* Comment.PreprocFile */
.dark .codehilite .c1 { color: #79d618; font-style: italic } /* Comment.Single */
.dark .codehilite .cs { color: #79d618; font-style: italic } /* Comment.Special */
.dark .codehilite .gd { color: #FF5370 } /* Generic.Deleted */
.dark .codehilite .ge { color: #89DDFF } /* Generic.Emph */
.dark .codehilite .gr { color: #FF5370 } /* Generic.Error */
.dark .codehilite .gh { color: #C3E88D } /* Generic.Heading */
.dark .codehilite .gi { color: #C3E88D } /* Generic.Inserted */
.dark .codehilite .go { color: #79d618 } /* Generic.Output */
.dark .codehilite .gp { color: #FFCB6B } /* Generic.Prompt */
.dark .codehilite .gs { color: #FF5370 } /* Generic.Strong */
.dark .codehilite .gu { color: #89DDFF } /* Generic.Subheading */
.dark .codehilite .gt { color: #FF5370 } /* Generic.Traceback */
.dark .codehilite .kc { color: #89DDFF } /* Keyword.Constant */
.dark .codehilite .kd { color: #BB80B3 } /* Keyword.Declaration */
.dark .codehilite .kn { color: #89DDFF; font-style: italic } /* Keyword.Namespace */
.dark .codehilite .kp { color: #89DDFF } /* Keyword.Pseudo */
.dark .codehilite .kr { color: #BB80B3 } /* Keyword.Reserved */
.dark .codehilite .kt { color: #BB80B3 } /* Keyword.Type */
.dark .codehilite .ld { color: #C3E88D } /* Literal.Date */
.dark .codehilite .m { color: #F78C6C } /* Literal.Number */
.dark .codehilite .s { color: #C3E88D } /* Literal.String */
.dark .codehilite .na { color: #BB80B3 } /* Name.Attribute */
.dark .codehilite .nb { color: #82AAFF } /* Name.Builtin */
.dark .codehilite .nc { color: #FFCB6B } /* Name.Class */
.dark .codehilite .no { color: #EEFFFF } /* Name.Constant */
.dark .codehilite .nd { color: #82AAFF } /* Name.Decorator */
.dark .codehilite .ni { color: #89DDFF } /* Name.Entity */
.dark .codehilite .ne { color: #FFCB6B } /* Name.Exception */
.dark .codehilite .nf { color: #82AAFF } /* Name.Function */
.dark .codehilite .nl { color: #82AAFF } /* Name.Label */
.dark .codehilite .nn { color: #FFCB6B } /* Name.Namespace */
.dark .codehilite .nx { color: #EEFFFF } /* Name.Other */
.dark .codehilite .py { color: #FFCB6B } /* Name.Property */
.dark .codehilite .nt { color: #FF5370 } /* Name.Tag */
.dark .codehilite .nv { color: #89DDFF } /* Name.Variable */
.dark .codehilite .ow { color: #89DDFF; font-style: italic } /* Operator.Word */
.dark .codehilite .pm { color: #89DDFF } /* Punctuation.Marker */
.dark .codehilite .w { color: #EEFFFF } /* Text.Whitespace */
.dark .codehilite .mb { color: #F78C6C } /* Literal.Number.Bin */
.dark .codehilite .mf { color: #F78C6C } /* Literal.Number.Float */
.dark .codehilite .mh { color: #F78C6C } /* Literal.Number.Hex */
.dark .codehilite .mi { color: #F78C6C } /* Literal.Number.Integer */
.dark .codehilite .mo { color: #F78C6C } /* Literal.Number.Oct */
.dark .codehilite .sa { color: #BB80B3 } /* Literal.String.Affix */
.dark .codehilite .sb { color: #C3E88D } /* Literal.String.Backtick */
.dark .codehilite .sc { color: #C3E88D } /* Literal.String.Char */
.dark .codehilite .dl { color: #EEFFFF } /* Literal.String.Delimiter */
.dark .codehilite .sd { color: #79d618; font-style: italic } /* Literal.String.Doc */
.dark .codehilite .s2 { color: #C3E88D } /* Literal.String.Double */
.dark .codehilite .se { color: #EEFFFF } /* Literal.String.Escape */
.dark .codehilite .sh { color: #C3E88D } /* Literal.String.Heredoc */
.dark .codehilite .si { color: #89DDFF } /* Literal.String.Interpol */
.dark .codehilite .sx { color: #C3E88D } /* Literal.String.Other */
.dark .codehilite .sr { color: #89DDFF } /* Literal.String.Regex */
.dark .codehilite .s1 { color: #C3E88D } /* Literal.String.Single */
.dark .codehilite .ss { color: #89DDFF } /* Literal.String.Symbol */
.dark .codehilite .bp { color: #89DDFF } /* Name.Builtin.Pseudo */
.dark .codehilite .fm { color: #82AAFF } /* Name.Function.Magic */
.dark .codehilite .vc { color: #89DDFF } /* Name.Variable.Class */
.dark .codehilite .vg { color: #89DDFF } /* Name.Variable.Global */
.dark .codehilite .vi { color: #89DDFF } /* Name.Variable.Instance */
.dark .codehilite .vm { color: #82AAFF } /* Name.Variable.Magic */
.dark .codehilite .il { color: #F78C6C } /* Literal.Number.Integer.Long */

